@@ -49,14 +49,20 @@
 
 package com.openexchange.mobile.api.facade.configuration;
 
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.Iterator;
+import java.util.LinkedList;
+import java.util.List;
 import java.util.Map;
 
 import javax.servlet.http.HttpServletRequest;
 
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.mobile.api.facade.configuration.clientspecific.MatchingHostConfiguration;
 import com.openexchange.mobile.api.facade.utils.RequestUtil;
+import com.openexchange.mobile.api.facade.utils.UserAgentInfo;
 
 import lombok.experimental.UtilityClass;
 
@@ -69,22 +75,45 @@ public class ConfigurationHelper {
 
     private HostConfiguration defaultHostConfiguration;
 
-    private Map<String, HostConfiguration> configurations = Collections.emptyMap();
+    private Map<String, List<HostConfiguration>> configurations = Collections.emptyMap();
 
     public void initialize(ConfigurationService configurationService) {
         GlobalConfiguration.initialize(configurationService);
         defaultHostConfiguration = new HostConfiguration(configurationService);
-        configurations = loadConfigurations(configurationService, ConfigurationHelper.CONFIG_MOBILE_API_FACADE_CONFIG_YAML);
+        loadConfigurations(configurationService, ConfigurationHelper.CONFIG_MOBILE_API_FACADE_CONFIG_YAML);
     }
 
     public HostConfiguration getConfiguration(HttpServletRequest request) {
         String hostname = RequestUtil.getHostHeader(request);
-        return getConfiguration(hostname);
+        String xHostname = RequestUtil.getXHostHeader(request);
+        String userAgent = RequestUtil.getUserAgent(request);
+        UserAgentInfo userAgentInfo = RequestUtil.parseUserAgent(userAgent);
+        HostConfiguration hostConfiguration = getConfiguration(hostname, xHostname, userAgentInfo);
+        RequestUtil.checkUserAgent(userAgentInfo, hostConfiguration);
+        return hostConfiguration;
     }
 
-    public HostConfiguration getConfiguration(String hostname) {
-        HostConfiguration configuration = configurations.get(hostname);
-        return configuration != null ? configuration : defaultHostConfiguration;
+    public HostConfiguration getConfiguration(String hostname, String xHostname, UserAgentInfo userAgentInfo) {
+        if (xHostname != null) {
+            hostname = xHostname;
+        }
+        List<HostConfiguration> configurationList = configurations.get(hostname);
+        if (configurationList == null) {
+            return defaultHostConfiguration;
+        }
+        Iterator<HostConfiguration> it = configurationList.iterator();
+        while (it.hasNext()) {
+            HostConfiguration configuration = it.next();
+            if (configuration instanceof MatchingHostConfiguration) {
+                MatchingHostConfiguration matchingConfiguration = (MatchingHostConfiguration) configuration;
+                if (matchingConfiguration.matches(userAgentInfo)) {
+                    return matchingConfiguration;
+                }
+            } else {
+                return configuration;
+            }
+        }
+        return defaultHostConfiguration;
     }
 
     public HostConfiguration getDefaultConfiguration() {
@@ -139,31 +168,52 @@ public class ConfigurationHelper {
         return configurationService.getProperty(propertyName);
     }
 
-    public Map<String, HostConfiguration> loadConfigurations(ConfigurationService configurationService, String filename) {
+    public Map<String, List<HostConfiguration>> loadConfigurations(ConfigurationService configurationService, String filename) {
         Object yaml = configurationService.getYaml(filename);
         if (yaml instanceof Map) {
             @SuppressWarnings("unchecked")
             Map<String, Object> map = (Map<String, Object>) yaml;
             if (!map.isEmpty()) {
-                Map<String, HostConfiguration> configurations = parseHostConfigurations(map, defaultHostConfiguration);
-                if (!configurations.isEmpty()) {
-                    return configurations;
+                Map<String, List<HostConfiguration>> parsedConfigurations = parseHostConfigurations(map, defaultHostConfiguration);
+                if (!parsedConfigurations.isEmpty()) {
+                    configurations = parsedConfigurations;
+                    return parsedConfigurations;
                 }
             }
         }
-        return Collections.emptyMap();
+        configurations = Collections.emptyMap();
+        return configurations;
     }
 
-    private Map<String, HostConfiguration> parseHostConfigurations(Map<String, Object> map, HostConfiguration defaultConfiguration) {
-        Map<String, HostConfiguration> configurations = new HashMap<>();
+    private Map<String, List<HostConfiguration>> parseHostConfigurations(Map<String, Object> map, HostConfiguration defaultConfiguration) {
+        Map<String, List<HostConfiguration>> configurations = new HashMap<>();
         for (Map.Entry<String, Object> entry : map.entrySet()) {
             String name = entry.getKey();
-            if (entry.getValue() instanceof Map) {
+            Object value = entry.getValue();
+            if (value instanceof Map) {
                 @SuppressWarnings("unchecked")
-                Map<String, Object> properties = (Map<String, Object>) entry.getValue();
+                Map<String, Object> properties = (Map<String, Object>) value;
                 HostConfiguration configuration = new HostConfiguration(defaultConfiguration);
                 configuration.loadProperties(properties);
-                configurations.put(name, configuration);
+                List<HostConfiguration> configurationList = new ArrayList<>(1);
+                configurationList.add(configuration);
+                configurations.put(name, configurationList);
+            } else if (value instanceof List) {
+                List<HostConfiguration> configurationList = new LinkedList<>();
+                configurations.put(name, configurationList);
+                @SuppressWarnings("unchecked")
+                List<Map<String, Object>> list = (List<Map<String, Object>>) value;
+                Iterator<Map<String, Object>> it = list.iterator();
+                while (it.hasNext()) {
+                    Object rawListEntry = it.next();
+                    if (rawListEntry instanceof Map) {
+                        @SuppressWarnings("unchecked")
+                        Map<String, Object> properties = (Map<String, Object>) rawListEntry;
+                        MatchingHostConfiguration configuration = new MatchingHostConfiguration(defaultConfiguration);
+                        configuration.loadProperties(properties);
+                        configurationList.add(configuration);
+                    }
+                }
             }
         }
         return configurations;
