@@ -83,7 +83,6 @@ import org.apache.http.impl.client.BasicCookieStore;
 import org.apache.http.impl.client.CloseableHttpClient;
 import org.apache.http.impl.client.HttpClients;
 import org.apache.http.ssl.SSLContextBuilder;
-import org.apache.http.util.TextUtils;
 
 @UtilityClass
 public class RequestUtil {
@@ -122,7 +121,7 @@ public class RequestUtil {
 
     private static final int USER_AGENT_MAXLEN = 100;
 
-    private static final Pattern USER_AGENT_PATTERN = Pattern.compile("^(.*)\\.(.*)\\.Mail/(\\S*) \\(.*\\)$");
+    private static final Pattern USER_AGENT_PATTERN = Pattern.compile("^(.*)\\.(.*)\\.Mail/(\\S*) \\(OS:(.*); device:(.*)\\)$");
 
     private static final String HEADER_CACHE_CONTROL = "Cache-control";
 
@@ -179,8 +178,9 @@ public class RequestUtil {
         String trackingId = getTrackingId(request);
         String contextId = getContextId(request);
         String userId = getUserId(request);
-        HostConfiguration hostConfiguration = ConfigurationHelper.getConfiguration(request);
-        checkUserAgent(userAgent, hostConfiguration);
+        UserAgentInfo userAgentInfo = parseUserAgent(userAgent);
+        HostConfiguration hostConfiguration = ConfigurationHelper.getConfiguration(hostname, xHostname, userAgentInfo);
+        checkUserAgent(userAgentInfo, hostConfiguration);
         return enhanceSessionData(request, hostConfiguration, hostname, xHostname, userAgent, trackingId, contextId, userId, new SessionData(session, cookies));
     }
 
@@ -215,11 +215,7 @@ public class RequestUtil {
         return userAgent;
     }
 
-    public void checkUserAgent(String userAgent, HostConfiguration hostConfiguration) {
-        if (Strings.isEmpty(userAgent)) {
-            throw ApiFacadeException.missingHeader(HEADER_USER_AGENT);
-        }
-        UserAgentInfo userAgentInfo = parseUserAgent(userAgent);
+    public void checkUserAgent(UserAgentInfo userAgentInfo, HostConfiguration hostConfiguration) {
         if (!hostConfiguration.isAllowedBrand(userAgentInfo.getBrand())) {
             throw ApiFacadeException.forbidden();
         }
@@ -239,7 +235,7 @@ public class RequestUtil {
         }
         Semver clientVersion = new Semver(clientVersionStr, SemverType.LOOSE);
         Semver minimumClientVersion = new Semver(minimumClientVersionStr, SemverType.LOOSE);
-        return !clientVersion.isLowerThan(minimumClientVersion);
+        return minimumClientVersion.isLowerThanOrEqualTo(clientVersion);
     }
 
     private String getMinimumClientVersion(Map<String, String> minimumClientVersions, UserAgentInfo userAgentInfo) {
@@ -308,20 +304,25 @@ public class RequestUtil {
     }
 
     public UserAgentInfo parseUserAgent(String userAgent) {
-        if (TextUtils.isEmpty(userAgent) || userAgent.length() > USER_AGENT_MAXLEN) {
+        if (Strings.isEmpty(userAgent)) {
+            throw ApiFacadeException.missingHeader(HEADER_USER_AGENT);
+        }
+        if (userAgent.length() > USER_AGENT_MAXLEN) {
             throw ApiFacadeException.invalidUserAgent();
         }
         Matcher matcher = USER_AGENT_PATTERN.matcher(userAgent);
         if (!matcher.find()) {
             throw ApiFacadeException.invalidUserAgent();
         }
-        if (matcher.groupCount() != 3) {
+        if (matcher.groupCount() != 5) {
             throw ApiFacadeException.invalidUserAgent();
         }
         String brand = matcher.group(1);
         String platform = matcher.group(2);
         String clientVersion = matcher.group(3);
-        return new UserAgentInfo(brand, platform, clientVersion);
+        String osVersion = matcher.group(4).trim();
+        String device = matcher.group(5).trim();
+        return new UserAgentInfo(brand, platform, clientVersion, osVersion, device);
     }
 
     /*private*/ String getIfNoneMatch(HttpServletRequest request) {
